"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_generator_1 = require("./types.generator");
var type_analyser_1 = require("../utilities/type.analyser");
var fragments_generator_1 = require("./fragments.generator");
exports.createMethodsArgs = function (field, prefix, suffix) {
    var _a = field.args.reduce(function (acc, arg) {
        var argName = arg.name;
        var type = types_generator_1.generateQGLArg(arg);
        acc.GQLVariables.push("$" + argName + ": " + type);
        acc.GQLArgs.push(argName + ": $" + argName);
        return acc;
    }, {
        GQLVariables: [],
        GQLArgs: [],
    }), GQLVariables = _a.GQLVariables, GQLArgs = _a.GQLArgs;
    var methodArgsType = '';
    if (field.args.length) {
        var parsedSuffix = 'Args' + (suffix ? suffix : '');
        methodArgsType = "" + (prefix ? prefix : '') + field.name + parsedSuffix;
    }
    return {
        GQLArgs: GQLArgs,
        GQLVariables: GQLVariables,
        methodArgsType: methodArgsType,
    };
};
exports.createGraphQLFunction = function (_a) {
    var field = _a.field, ObjectTypes = _a.ObjectTypes, prefix = _a.prefix, suffix = _a.suffix, type = _a.type, scalarList = _a.scalarList;
    var hasArgs = field.args.length > 0;
    var methodName = field.name;
    var fragmentDisplay = "${isString ? fragment : '...' + fragmentName}";
    var _b = exports.createMethodsArgs(field, prefix, suffix), GQLArgs = _b.GQLArgs, GQLVariables = _b.GQLVariables, methodArgsType = _b.methodArgsType;
    var _c = type_analyser_1.evaluateType(field), isScalar = _c.isScalar, isEnum = _c.isEnum, typeName = _c.typeName;
    var returnedTypeDisplay = types_generator_1.getOneTSTypeDisplay({ field: field, prefix: prefix, suffix: suffix, scalarList: scalarList });
    var renderedFragmentInner = fragmentDisplay;
    if (!isScalar && !isEnum && type_analyser_1.isReturnTypeEdge(ObjectTypes, typeName)) {
        renderedFragmentInner = fragments_generator_1.createConnectionFragment(typeName, ObjectTypes, fragmentDisplay);
    }
    if (isScalar) {
        return "\n    const " + type.little + " = graphQlTag`\n      " + type.little + " " + methodName + " " + (hasArgs ? "(" + GQLVariables.join(',') + ")" : '') + " {\n        " + methodName + (hasArgs ? "(" + GQLArgs.join(',') + ")" : '') + "\n      }`,\n    return abortable" + type.high + "(" + type.little + ", " + hasArgs + ") as any;\n  ";
    }
    else {
        return "\n    return {\n      $fragment: (fragment: string | DocumentNode) => {\n        const { isString, isFragment, fragmentName } = guessFragmentType(fragment);\n        const " + type.little + " = graphQlTag`\n          " + type.little + " " + methodName + " " + (hasArgs ? "(" + GQLVariables.join(',') + ")" : '') + " {\n            " + methodName + (hasArgs ? "(" + GQLArgs.join(',') + ")" : '') + " {\n              " + renderedFragmentInner + "\n            }\n          } ${isFragment? fragment: ''}\n          `\n\n        return abortable" + type.high + "(" + type.little + ", " + hasArgs + ") as any;\n      }\n    }\n  ";
    }
};
exports.buildMethod = function (field, type, prefix, suffix, ObjectTypes, scalarList) {
    var hasArgs = field.args.length > 0;
    var methodName = field.name;
    var isScalar = type_analyser_1.evaluateType(field).isScalar;
    var methodArgsType = exports.createMethodsArgs(field, prefix, suffix).methodArgsType;
    var returnedTypeDisplay = types_generator_1.getOneTSTypeDisplay({ field: field, prefix: prefix, suffix: suffix, scalarList: scalarList });
    var genereratedFunction = exports.createGraphQLFunction({
        ObjectTypes: ObjectTypes,
        field: field,
        prefix: prefix,
        suffix: suffix,
        type: type,
        scalarList: scalarList,
    });
    var withArgs = hasArgs
        ? type_analyser_1.areAllArgsOptional(field.args)
            ? 'WithOptionalArgs'
            : 'WithArgs'
        : '';
    var template = "\n    " + methodName + "(): " + (isScalar
        ? "Abordable" + type.high + withArgs + "<" + returnedTypeDisplay + (hasArgs ? ',' + methodArgsType : '') + ">"
        : "Fragmentable" + type.high + withArgs + "<" + returnedTypeDisplay + (hasArgs ? ',' + methodArgsType : '') + ">") + " {\n        " + genereratedFunction + "\n      }\n    ,";
    return template;
};
var lol = {
    me: function () {
        return '';
    },
};
