"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var type_analyser_1 = require("../utilities/type.analyser");
exports.generateEnumType = function (object, prefix, suffix) {
    var ObjectName = object.name;
    var generatedFields = object.enumValues.map(function (field) {
        return "| '" + field.name + "'";
    });
    return "type " + (prefix ? prefix : '') + ObjectName + (suffix ? suffix : '') + " = \n        " + generatedFields.join('\n') + "\n    ";
};
exports.getOneTSTypeDisplay = function (_a) {
    var field = _a.field, prefix = _a.prefix, suffix = _a.suffix, scalarList = _a.scalarList;
    var _b = type_analyser_1.evaluateType(field), isScalar = _b.isScalar, typeName = _b.typeName, isArray = _b.isArray;
    return isScalar
        ? scalarList[typeName]
        : "" + (prefix ? prefix : '') + typeName + (suffix ? suffix : '') + (isArray ? '[]' : '');
};
exports.generatedTsFields = function (fields, prefix, suffix, scalarList) {
    return fields.map(function (field) {
        var propertyName = field.name;
        var isOptional = type_analyser_1.evaluateType(field).isOptional;
        var TStypeName = exports.getOneTSTypeDisplay({ field: field, prefix: prefix, suffix: suffix, scalarList: scalarList });
        return "" + propertyName + (isOptional ? '?' : '') + ": " + TStypeName + ";";
    });
};
exports.generateQGLArg = function (field) {
    var _a = type_analyser_1.evaluateType(field), isArray = _a.isArray, isArrayOptional = _a.isArrayOptional, isOptional = _a.isOptional, typeName = _a.typeName;
    return "" + (isArray ? '[' : '') + typeName + (isOptional ? '' : '!') + (isArray ? ']' : '') + (isArrayOptional ? '!' : '');
};
exports.getObjectTSInterfaces = function (object, prefix, suffix, scalarList) {
    var ObjectName = object.name;
    var fieldsKey = object.kind === 'OBJECT' || object.kind === 'INTERFACE' ? 'fields' : 'inputFields';
    var generatedFields = exports.generatedTsFields(object[fieldsKey], prefix, suffix, scalarList);
    return exports.buildTsInterfaceString(ObjectName, generatedFields, prefix, suffix);
};
exports.getQueriesArgsTSInterfaces = function (object, prefix, suffix, scalarList) {
    var ObjectName = object.name;
    var parsedSuffix = 'Args' + (suffix ? suffix : '');
    var generatedFields = exports.generatedTsFields(object.args, prefix, suffix, scalarList);
    return exports.buildTsInterfaceString(ObjectName, generatedFields, prefix, parsedSuffix);
};
exports.buildTsInterfaceString = function (name, fields, prefix, suffix) {
    return "interface " + (prefix ? prefix : '') + name + (suffix ? suffix : '') + " {\n    " + fields.join('\n') + "\n  }\n";
};
